/*
 * color.c in Virtual Combat
 * Copyright (c) 1993 by David Jameson
 *
 * Permission to use, copy, modify and distribute this software
 * for any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission appear in the supporting
 * documentation.
 */


/*
 * Definitions of colours used in game.
 * There are a small number of base colours, each of which can
 * be displayed at various brightnesses, to allow for light-sourced
 * shading.
 */

#include "defs.h"

/* Set up an XColor structure when given RGB values in the range 0.0 to 1.0 */
void set_xcolor(disp *d, XColor *col, float r, float g, float b)
{
    col->red = (unsigned short)(MAXCOLOR * r);
    col->green = (unsigned short)(MAXCOLOR * g);
    col->blue = (unsigned short)(MAXCOLOR * b);
    if (! XAllocColor(d->dpy, d->cmap, col)) {
        perror("XAllocColor()");
        exit(1);
    }
}  /* set_xcolor() */


/* Calculate RGB values and allocate colormap entries for a colour */
void setcol(disp *d, color *col, float r, float g, float b)
{
    int i;

    col->base.red = (unsigned short)(r * MAXCOLOR);
    col->base.green = (unsigned short)(g * MAXCOLOR);
    col->base.blue = (unsigned short)(b * MAXCOLOR);

    /* Set up colour shades.  Intensity ranges from (base/SHADES) to (base)
       (black is not defined for each colour). */

    for (i = 0; i < SHADES; i++) {
        col->shade[i].red = (col->base.red * (i + 1)) / SHADES;
        col->shade[i].green = (col->base.green * (i + 1)) / SHADES;
        col->shade[i].blue = (col->base.blue * (i + 1)) / SHADES;
        if (! XAllocColor(d->dpy, d->cmap, &col->shade[i])) {
            perror("XAllocColor()");
            exit(1);
        }
    }
}  /* setcol() */

