/*
 * defs.h in Virtual Combat
 * Copyright (c) 1993 by David Jameson
 *
 * Permission to use, copy, modify and distribute this software
 * for any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission appear in the supporting
 * documentation.
 */

/* Definitions common to both client and server programs */

#ifndef _DEFS_H_
#define _DEFS_H_

/* #define DEBUG 1 */  /* Uncomment to enable debugging */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>

#define SHADES 15  /* Number of shades (brightnesses) for each colour base */
#define MAXCOLOR 65535	/* Value which X uses to represent a maximum RGB
			   component */
#define DEFAULTX 400  /* Size of window */
#define DEFAULTY 400
#define MAXXSIZE 800
#define MAXYSIZE 800
#define MINXSIZE 260
#define MINYSIZE 260
#define PLANES 8  /* Number of bitplanes */
#define USE_FAST_SET_ECHO
#define HOSTFILE "host"   /* File containing address of machine where
			     game was started */
#define PORT_NUMBER 2002  /* Port number used for client-server link */
#define MAX_LOGINS 25	  /* Maximum number of people logged onto server */
#define MAX_PLAYERS 8	  /* Maximum number of people playing a game */
#define DATASIZE 65000	  /* Size of data buffers (not inc. headers) */
#define NAMELEN 20	  /* Max length of a player's name */
#define MAX_ENERGY 40	  /* Starting value for energy */

typedef struct {
    Display *dpy;
    GC gc;
    Window w;	     /* Game window */
    Window root;     /* Root window of screen */
    Pixmap pix;      /* Pixmap used as buffer */
    Colormap cmap;   /* Colormap for our window */
    Visual *visual;  /* Visual type for screen */
} disp;


typedef struct {  /* Update of position & orientation data, sent to server */
    long x, y, z, yrot;
    unsigned char id;  /* id of client who sent this info */
} update;


typedef struct _color {
    XColor base;
    XColor shade[SHADES];
} color;

#ifdef USE_FAST_SET_ECHO
#define USE_OLD_TTY
#include <sys/ioctl.h>
#undef USE_OLD_TTY

#define local_echo_on() \
do { \
    struct sgttyb tty; \
    ioctl(fileno(stdin), TIOCGETP, &tty); \
    tty.sg_flags |= ECHO; \
    ioctl(fileno(stdin), TIOCSETP, &tty); \
} while (0)

#define local_echo_off() \
do { \
    struct sgttyb tty; \
    ioctl(fileno(stdin), TIOCGETP, &tty); \
    tty.sg_flags &= ~ECHO; \
    ioctl(fileno(stdin), TIOCSETP, &tty); \
} while (0)    /* The only way to make a function into a macro */
#else
#define local_echo_on() system("stty echo")
#define local_echo_off() system("stty -echo")
#endif


#ifndef FALSE
#define FALSE		0		/* boolean false */
#define TRUE		(!FALSE)        /* boolean true */
#endif

#define POSITIVE 1
#define NEGATIVE (-1)

#define min(a, b) ((a) < (b) ? (a) : (b))
#define max(a, b) ((a) > (b) ? (a) : (b))

#define sign(x) ((x) < 0 ? NEGATIVE : POSITIVE)


typedef struct {
    unsigned short length;  /* Total length of packet
			       including header */
    unsigned char pkttype;  /* Type of packet */
    unsigned char data[DATASIZE];
} databuffer;

#define HDRSIZE (sizeof(unsigned short) + sizeof(unsigned char))

/* prototypes for external scope functions */

extern void bcopy(void *b1, void *b2, int length);
extern int bind(int s, struct sockaddr *name, int namelen);
extern void bzero(void *b, int length);
extern int setsockopt(int s, int level, int optname, char *optval, int optlen);
extern int socket(int domain, int type, int protocol);
extern int accept(int s, struct sockaddr *addr, int *addrlen);
extern int listen(int s, int backlog);
extern int connect(int s, struct sockaddr *name, int namelen);
extern int getdtablesize(void);
extern int select(int width, fd_set *readfds, fd_set *writefds,
    fd_set *exceptfds, struct timeval *timeout);
extern int setitimer(int which, struct itimerval *value,
    struct itimerval *ovalue);

extern Window open_window(Display *dpy, Window parent, int width, int height,
    int depth, char *name, char *name2);
extern disp *setup_display(char *dpyname, int x, int y, char *name, char *name2,
    int depth);

extern void set_xcolor(disp *d, XColor *col, float r, float g, float b);
extern void setcol(disp *d, color *col, float r, float g, float b);

#endif

