/*
 * graphics.c in Virtual Combat
 * Copyright (c) 1993 by David Jameson
 *
 * Permission to use, copy, modify and distribute this software
 * for any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission appear in the supporting
 * documentation.
 */


/* Routines for interfacing with X */

#include "defs.h"

static disp d;	/* Display information */

disp *setup_display (char *dpyname, int x, int y, char *name, char *name2,
	int depth)
{
    Display *dpy;
    Window win, w;
    GC gc;  /* Graphics context */
    int screen;
    Visual *visual;

    /* Try to open display */

    if (! dpyname) {
	if (! (dpy = XOpenDisplay(""))) {
	    printf("Error - can't open display\n");
	    exit(1);
	}
    }
    else if (! (dpy = XOpenDisplay(dpyname))) {
	printf("Error - can't open display\n");
	exit(1);
    }

    if (depth == 0) depth = DefaultDepth(dpy, DefaultScreen(dpy));

    d.dpy = dpy;
    screen = DefaultScreen(dpy);
    visual = DefaultVisual(dpy, screen);
    d.visual = visual;

    /* Find root window */

    if (! (win = RootWindow(dpy, screen))) {
	printf("Error - orphaned window\n");
	exit(1);
    }

    d.root = win;
    w = open_window(dpy, win, x, y, depth, name, name2);

    d.w = w;

    gc = XDefaultGC(dpy, screen);
    XSetGraphicsExposures(dpy, gc, 0);  /* Tell server not to send exposure
					   events */
    d.gc = gc;
    XSetPlaneMask(dpy, gc, AllPlanes);
    d.pix = XCreatePixmap(dpy, (Drawable)w, x, y, depth);
    d.cmap = XCreateColormap(dpy, w, visual, AllocNone);
    XSetWindowColormap(dpy, w, d.cmap);
    return &d;
}  /* setup_display */


/* Open up a window */
Window open_window(Display *dpy, Window parent, int width, int height,
	int depth, char *name, char *name2)
{
    XSetWindowAttributes attr;
    XSizeHints xshints;
    int screen;
    long mask;
    Window w;
    XWMHints myhints;

    screen = DefaultScreen(dpy);
    attr.border_pixel = WhitePixel(dpy, screen);
    attr.background_pixel = BlackPixel(dpy, screen);
    attr.event_mask =	KeyPressMask | KeyReleaseMask /*| ResizeRedirectMask */
		       | ButtonPressMask | ButtonReleaseMask | ExposureMask |
		       FocusChangeMask;
    mask = CWBackPixel | CWBorderPixel | CWEventMask;

    if (! (w = XCreateWindow(dpy, parent, 0, 0, width, height, 0, depth,
			     InputOutput, CopyFromParent, mask, &attr))) {
	perror("Error opening window");
	exit(1);
    }

    myhints.input = True;
    myhints.flags = InputHint;
    XSetWMHints(dpy, w, &myhints);

    xshints.flags = PMinSize | PMaxSize;
    xshints.max_width = MAXXSIZE;
    xshints.max_height = MAXYSIZE;
    /*xshints.width = x;
    xshints.height = y;*/
    xshints.min_width = MINXSIZE;
    xshints.min_height = MINYSIZE;
    /*xshints.x = 0;
    xshints.y = 0;*/
    XSetStandardProperties(dpy, w, name, name2, 0, NULL, 0, &xshints);
    XMapRaised(dpy,w);
    /*XNextEvent(dpy, &event);
    if (event.type == ClientMessage) {
	e = (XClientMessageEvent *) &event;
	mname = XGetAtomName(dpy,e->message_type);
	if (mname) XFree(mname);
    }
    else fprintf(stderr, "error - unexpected event.\n");*/

    return w;
}

